#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#include <windows.h>
#include <assert.h>
#include <iostream>
using namespace std;

#include <gl/glew.h>
#include <gl/glu.h>
#include <gl/glut.h>

#include <Cg/cg.h>
#include <Cg/cggl.h>
#include <imdebuggl.h>

#include <Vec3f.hpp>
#include <Vec2f.hpp>

#include "RT/RenderTexture.h"
#include "pt_site.hpp"

#include <vector>
using namespace std;

CGprogram cgPoint;
CGcontext cgContext;
CGprofile cgFragmentProfile;
CGparameter cgResParam, cgPtCoordParam;

RenderTexture *render_tgt;
GLuint tex_name;

int ResX=512, ResY=512;
//Vec2f w_min(0,0);
//Vec2f w_max(1,1);
float MaxDist = 1.0f;

vector <PointSite*> points;

#define CG_CUR_PROFILE CG_PROFILE_ARBFP1

void SetupCG()
{
  cgContext = cgCreateContext();
  
  assert(cgContext);
  
  cgFragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
  
  assert(cgFragmentProfile != CG_PROFILE_UNKNOWN);
  cgGLSetOptimalOptions(cgFragmentProfile);	
  
  assert(cgGLIsProfileSupported(CG_CUR_PROFILE));
  
  cgPoint = cgCreateProgramFromFile(cgContext, CG_SOURCE, "CG/ps.cg", CG_CUR_PROFILE, "point_dist", 0);
  
  if(!cgPoint)
  {
    CGerror Error = cgGetError();
    cout << cgGetErrorString(Error) << endl;
    exit(0);    
  }
  
  cgGLLoadProgram(cgPoint);
  
  cgResParam    = cgGetNamedParameter(cgPoint, "inv_res");
  cgPtCoordParam= cgGetNamedParameter(cgPoint, "pt_coord");
  assert(cgResParam && cgPtCoordParam);

  assert(cgContext);

}

void FreeSites()
{
  for (unsigned i=0;i<points.size();i++)
  {
    delete points[i];
  }
}

void SetupSites(unsigned  num_pts)
{
  //srand(7);
  for (unsigned i=0;i<num_pts;i++)
  {
    Vec2r pos(RAND(), RAND());
    printf("Added point (%f,%f)\n",pos.x, pos.y);
    PointSite* pt = new PointSite(pos);
    pt->SetID(0, i);
    points.push_back(pt);
  }
}

void ComputeDistances()
{
  float res[2] = { 1.0f / ResX, 1.0f / ResY};
  float pt_coord[2] = {0.5, 0.5};

  cgGLEnableProfile(cgFragmentProfile);
  cgGLBindProgram(cgPoint);
  cgGLSetParameter2fv(cgResParam, res);

  unsigned char color[4];
  // compute distance function for each site
  for (unsigned i=0; i<points.size(); i++)
  {
    vdINT2RGBA(points[i]->GetSiteID(), color);
    glColor4ubv(color);
    points[i]->ComputeDist(MaxDist);
  }

  cgGLDisableProfile(cgFragmentProfile);
}

void Exit()
{
  FreeSites();
  exit(1);
}

void myDisplay(void)
{
  render_tgt->BeginCapture();
  ComputeDistances();

  imdebugPixelsf(0,0,ResX,ResY,GL_RGB);
  imdebugDepthf(0,0,ResX,ResY);

  render_tgt->EndCapture();
  Exit();
}

void init_rt()
{
  render_tgt = new RenderTexture(ResX, ResY, true, true, false);
  render_tgt->Initialize(true, true, false, false, false, 32, 32, 32, 32);
  
  render_tgt->BeginCapture();


  glMatrixMode(GL_PROJECTION); 
  glLoadIdentity();
  glOrtho(0.0, 1.0, 0.0, 1.0, 0.0, -MaxDist);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glViewport(0,0,ResX,ResY);

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glEnable(GL_DEPTH_TEST);
  glDepthMask(GL_TRUE);
  glDisable(GL_CULL_FACE);

  render_tgt->EndCapture();
}

void init_gl()
{
  GLenum err = glewInit();
  if (GLEW_OK != err)
  {
    /* Problem: glewInit failed, something is seriously wrong. */
    fprintf(stderr, "Error: %s\n", glewGetErrorString(err));
    exit(1);
  }
  fprintf(stdout, "Status: Using GLEW %s\n", glewGetString(GLEW_VERSION));	

}

void Init()
{
  init_gl();
  SetupCG();
  init_rt();
  SetupSites(5);
}

void main(void)
{
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_ALPHA);
  glutInitWindowSize(512,512);
  glutInitWindowPosition(200,100);
  glutCreateWindow("Distance Field Test");

  Init();

  glutDisplayFunc(myDisplay);

  glEnable(GL_DEPTH_TEST);

  glutMainLoop();
}
